@@ -155,24 +155,37 @@ dialog_destroyed (gpointer data, GObject *where_object_was)
 	gtk_object_destroy (GTK_OBJECT (ecmld));
 }
 
+static void
+build_list_callback (int row, gpointer data) {
+	GList **listp = data;
+
+	*listp = g_list_prepend (*listp, GINT_TO_POINTER (row));
+}
+
 static void
 dialog_response (GtkObject *dialog, int id, ECategoriesMasterListDialog *ecmld)
 {
 	GtkWidget *scrolled;
+	
 	switch (id) {
 	case 0:    /* Remove */
 		scrolled = glade_xml_get_widget (ecmld->priv->gui, "custom-etable");
 		if (scrolled && E_IS_TABLE_SCROLLED (scrolled)) {
 			ETable *table;
-			int row;
-
+			GList *list = NULL;
+			GList *l;
+	
 			table = e_table_scrolled_get_table (E_TABLE_SCROLLED(scrolled));
 
-			row = e_table_get_cursor_row(table);
-			if (row != -1) {
+			e_table_selected_row_foreach (table, build_list_callback, &list);
+			for (l = list; l != NULL; l = g_list_next (l)) {
+				int row = GPOINTER_TO_INT (l->data);
+
 				e_categories_master_list_delete (ecmld->priv->ecml, row);
-				e_categories_master_list_commit (ecmld->priv->ecml);
 			}
+			g_list_free (list);
+			
+			e_categories_master_list_commit (ecmld->priv->ecml);
 		}
 		break;
 	case GTK_RESPONSE_CLOSE:
@@ -181,6 +194,8 @@ dialog_response (GtkObject *dialog, int id, ECategoriesMasterListDialog *ecmld)
 	}
 }
 
+
+
 #if 0
 static void
 connect_button (ECategoriesMasterListDialog *ecmld, GladeXML *gui, const char *widget_name, void *cback)
